# -*- coding: utf-8 -*-
import datetime
from alerts import Alerts
from constants import CLUSTER_SIZE_MISMATCH_RETRY_TIME
from constants import REPL_FACTOR_MISMATCH_RETRY_TIME


class ClusterLevel(Alerts):

    def __init__(self, cluster_id):

        Alerts.__init__(self, cluster_id)
        self.attr_to_handler = (
            ('cluster_integrity', 'on_cluster_integrity_change')
        )

        self.cluster_integrity_alert_data = dict(status="green", last_id=None)
        self.cluster_size_alert_data = dict(status="green", last_id=None, last_cluster_size_mismatch_timestamp=None)
        self.replication_factor_mismatch_alert_data = dict()

    def on_cluster_integrity_change(self, cluster_integrity):

        messages = dict(
            red="Cluster integrity has been disturbed",
            green="Cluster integrity has been restored"
        )

        if not cluster_integrity:
            return

        alert_status = self.get_alert_status(
            cluster_integrity == "false",  # Red alert condition
            None                           # Yellow alert condition
        )

        if alert_status != self.cluster_integrity_alert_data['status']:
            self.cluster_integrity_alert_data['status'] = alert_status
            self.cluster_integrity_alert_data['last_id'] = self.create_alert(
                messages[alert_status], alert_status, None, "alert", self.cluster_integrity_alert_data['last_id'])

    def on_cluster_size_change(self, cluster_size_is_same):

        messages = dict(
            red="Mismatch in the cluster sizes as shown by nodes",
            green="Cluster sizes as shown by nodes match now"
        )

        alert_status = self.get_alert_status(
            cluster_size_is_same is False,  # Red alert condition
            None                            # Yellow alert condition
        )

        if alert_status != self.cluster_size_alert_data['status']:
            if alert_status == "red":
                time_now = datetime.datetime.now()
                if self.cluster_size_alert_data['last_cluster_size_mismatch_timestamp'] == None:
                    self.cluster_size_alert_data['last_cluster_size_mismatch_timestamp'] = time_now
                    return
                elif (time_now - self.cluster_size_alert_data['last_cluster_size_mismatch_timestamp']).seconds < CLUSTER_SIZE_MISMATCH_RETRY_TIME:
                    return

            self.cluster_size_alert_data['status'] = alert_status
            self.cluster_size_alert_data['last_id'] = self.create_alert(
                messages[alert_status], alert_status, None, "alert", self.cluster_size_alert_data['last_id'])

    def on_new_node_add(self, node_address):

        message = "Node <strong>%s</strong> is added to the cluster"
        self.create_alert(message % (node_address), "green", node_address, "alert")

    def on_new_node_add_but_down(self, node_address):

        message = "Node <strong>%s</strong> is added to the cluster, currently not started"
        self.create_alert(message % (node_address), "green", node_address, "alert")

    def on_different_cluster_node_add(self, node_address):

        message = "Node <strong>%s</strong> cannot be monitored here as it belongs to a different cluster."
        self.create_alert(message % (node_address), "green", None, "notification")

    def on_backup_success(self, namespace, destination_node_address, destination_location):

        message = "Namespace <strong>%s</strong> successfully backed up to location %s:%s" % (namespace, destination_node_address, destination_location)
        self.create_alert(message, "green", None, "alert")

    def on_restore_success(self, namespace, source_node_address, source_location):

        if namespace != "":
            message = "Namespace <strong>%s</strong> successfully restored from location %s:%s" % (namespace, source_node_address, source_location)
        else:
            message = "Namespace from backup file successfully restored from location %s:%s" % (source_node_address, source_location)
        self.create_alert(message, "green", None, "alert")

    def on_replication_factor_change(self, replication_factor_is_same, namespace_name):

        messages = dict(
            red="Mismatch in replication factor for namespace %s as shown by nodes" % namespace_name,
            green="Replication factor for namespace %s as shown by nodes match now" % namespace_name
        )

        alert_status = self.get_alert_status(
            replication_factor_is_same is False,  # Red alert condition
            None                            # Yellow alert condition
        )

        if namespace_name not in self.replication_factor_mismatch_alert_data.keys():
            self.replication_factor_mismatch_alert_data[namespace_name] = dict(status="green", last_id=None, last_replication_factor_mismatch_timestamp=None)
        if alert_status != self.replication_factor_mismatch_alert_data[namespace_name]['status']:
            time_now = datetime.datetime.now()
            if self.replication_factor_mismatch_alert_data[namespace_name]['last_replication_factor_mismatch_timestamp'] is None:
                self.replication_factor_mismatch_alert_data[namespace_name]['last_replication_factor_mismatch_timestamp']  = time_now
                return
            elif (time_now - self.replication_factor_mismatch_alert_data[namespace_name]['last_replication_factor_mismatch_timestamp'] ).seconds < REPL_FACTOR_MISMATCH_RETRY_TIME:
                return

            self.replication_factor_mismatch_alert_data[namespace_name]['status'] = alert_status
            self.replication_factor_mismatch_alert_data[namespace_name]['last_id'] = self.create_alert(
            messages[alert_status], alert_status, None, "alert",
            self.replication_factor_mismatch_alert_data[namespace_name]['last_id'])
        self.replication_factor_mismatch_alert_data[namespace_name]['last_replication_factor_mismatch_timestamp'] = None
